-----------------------------------------------------------
-- FPGA Training Board(B6101)
-- 2Color 16x16 Dot Matrix LED Controller
-----------------------------------------------------------
-- File   : LedCont_vhdl_top.vhd
-- Date   : 2007-11-28
-- Family : Cyclone
-- Device : EP1C3T144C8
--
-- Designed by TOYO-LINX Co.,Ltd. / yKikuchi
-----------------------------------------------------------

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;

entity LedCont_vhdl_top is
	port(
		CLK			:	in std_logic;
		LED_SEL		:	out std_logic_vector(15 downto 0);
		LED_DATA_R	:	out std_logic_vector(0 to 15);
		LED_DATA_G	:	out std_logic_vector(0 to 15);
		DATA		:	in std_logic_vector(7 downto 0);
		ADDR		:	in std_logic_vector(5 downto 0);
		WR_N		:	in std_logic;
		CS_N		:	in std_logic;
		SW_IN		:	in std_logic_vector(6 downto 0);
		SW_OUT		:	out std_logic_vector(6 downto 0);
		BUZZ_IN		:	in std_logic;
		BUZZ_OUT	:	out std_logic
	);
end LedCont_vhdl_top;

architecture RTL of LedCont_vhdl_top is

	subtype RAMWORD is std_logic_vector(7 downto 0);
	type RAMARRAY is array(0 to 63) of RAMWORD;
	signal data_reg	:	RAMARRAY;

	signal div_cnt	:	std_logic_vector(31 downto 0);
	signal div_clk	:	std_logic;
	signal scan_cnt	:	std_logic_vector(3 downto 0);

begin
	process(WR_N)
	begin
		if(WR_N'event and WR_N = '1')then
			if(CS_N = '0')then
				case ADDR is
					when "000000" => data_reg( 0) <= DATA;
					when "000001" => data_reg( 1) <= DATA;
					when "000010" => data_reg( 2) <= DATA;
					when "000011" => data_reg( 3) <= DATA;
					when "000100" => data_reg( 4) <= DATA;
					when "000101" => data_reg( 5) <= DATA;
					when "000110" => data_reg( 6) <= DATA;
					when "000111" => data_reg( 7) <= DATA;
					when "001000" => data_reg( 8) <= DATA;
					when "001001" => data_reg( 9) <= DATA;
					when "001010" => data_reg(10) <= DATA;
					when "001011" => data_reg(11) <= DATA;
					when "001100" => data_reg(12) <= DATA;
					when "001101" => data_reg(13) <= DATA;
					when "001110" => data_reg(14) <= DATA;
					when "001111" => data_reg(15) <= DATA;
					when "010000" => data_reg(16) <= DATA;
					when "010001" => data_reg(17) <= DATA;
					when "010010" => data_reg(18) <= DATA;
					when "010011" => data_reg(19) <= DATA;
					when "010100" => data_reg(20) <= DATA;
					when "010101" => data_reg(21) <= DATA;
					when "010110" => data_reg(22) <= DATA;
					when "010111" => data_reg(23) <= DATA;
					when "011000" => data_reg(24) <= DATA;
					when "011001" => data_reg(25) <= DATA;
					when "011010" => data_reg(26) <= DATA;
					when "011011" => data_reg(27) <= DATA;
					when "011100" => data_reg(28) <= DATA;
					when "011101" => data_reg(29) <= DATA;
					when "011110" => data_reg(30) <= DATA;
					when "011111" => data_reg(31) <= DATA;
					when "100000" => data_reg(32) <= DATA;
					when "100001" => data_reg(33) <= DATA;
					when "100010" => data_reg(34) <= DATA;
					when "100011" => data_reg(35) <= DATA;
					when "100100" => data_reg(36) <= DATA;
					when "100101" => data_reg(37) <= DATA;
					when "100110" => data_reg(38) <= DATA;
					when "100111" => data_reg(39) <= DATA;
					when "101000" => data_reg(40) <= DATA;
					when "101001" => data_reg(41) <= DATA;
					when "101010" => data_reg(42) <= DATA;
					when "101011" => data_reg(43) <= DATA;
					when "101100" => data_reg(44) <= DATA;
					when "101101" => data_reg(45) <= DATA;
					when "101110" => data_reg(46) <= DATA;
					when "101111" => data_reg(47) <= DATA;
					when "110000" => data_reg(48) <= DATA;
					when "110001" => data_reg(49) <= DATA;
					when "110010" => data_reg(50) <= DATA;
					when "110011" => data_reg(51) <= DATA;
					when "110100" => data_reg(52) <= DATA;
					when "110101" => data_reg(53) <= DATA;
					when "110110" => data_reg(54) <= DATA;
					when "110111" => data_reg(55) <= DATA;
					when "111000" => data_reg(56) <= DATA;
					when "111001" => data_reg(57) <= DATA;
					when "111010" => data_reg(58) <= DATA;
					when "111011" => data_reg(59) <= DATA;
					when "111100" => data_reg(60) <= DATA;
					when "111101" => data_reg(61) <= DATA;
					when "111110" => data_reg(62) <= DATA;
					when "111111" => data_reg(63) <= DATA;
					when others   => data_reg( 0) <= "XXXXXXXX";
				end case;
			end if;
		end if;
	end process;

	process(CLK)
	begin
		if(CLK'event and CLK='1') then
			div_cnt <= div_cnt + 1;
		end if;
	end process;
	div_clk <= div_cnt(11);
	
	process(div_clk)
	begin
		if(div_clk'event and div_clk='1') then
			scan_cnt <= scan_cnt + 1;
		end if;
	end process;

	process(scan_cnt)
	begin
		case scan_cnt is
			when "0000" =>
				for I in 0 to 3 loop
					for J in 0 to 3 loop
						LED_DATA_R(I*4+J) <= data_reg(I)(6-J*2);
						LED_DATA_G(I*4+J) <= data_reg(I)(6-J*2+1);
					end loop;
				end loop;
				LED_SEL <= "1000000000000000";
			when "0001" =>
				for I in 0 to 3 loop
					for J in 0 to 3 loop
						LED_DATA_R(I*4+J) <= data_reg(I+4)(6-J*2);
						LED_DATA_G(I*4+J) <= data_reg(I+4)(6-J*2+1);
					end loop;
				end loop;
				LED_SEL <= "0100000000000000";
			when "0010" =>
				for I in 0 to 3 loop
					for J in 0 to 3 loop
						LED_DATA_R(I*4+J) <= data_reg(I+8)(6-J*2);
						LED_DATA_G(I*4+J) <= data_reg(I+8)(6-J*2+1);
					end loop;
				end loop;
				LED_SEL <= "0010000000000000";
			when "0011" =>
				for I in 0 to 3 loop
					for J in 0 to 3 loop
						LED_DATA_R(I*4+J) <= data_reg(I+12)(6-J*2);
						LED_DATA_G(I*4+J) <= data_reg(I+12)(6-J*2+1);
					end loop;
				end loop;
				LED_SEL <= "0001000000000000";
			when "0100" =>
				for I in 0 to 3 loop
					for J in 0 to 3 loop
						LED_DATA_R(I*4+J) <= data_reg(I+16)(6-J*2);
						LED_DATA_G(I*4+J) <= data_reg(I+16)(6-J*2+1);
					end loop;
				end loop;
				LED_SEL <= "0000100000000000";
			when "0101" =>
				for I in 0 to 3 loop
					for J in 0 to 3 loop
						LED_DATA_R(I*4+J) <= data_reg(I+20)(6-J*2);
						LED_DATA_G(I*4+J) <= data_reg(I+20)(6-J*2+1);
					end loop;
				end loop;
				LED_SEL <= "0000010000000000";
			when "0110" =>
				for I in 0 to 3 loop
					for J in 0 to 3 loop
						LED_DATA_R(I*4+J) <= data_reg(I+24)(6-J*2);
						LED_DATA_G(I*4+J) <= data_reg(I+24)(6-J*2+1);
					end loop;
				end loop;
				LED_SEL <= "0000001000000000";
			when "0111" =>
				for I in 0 to 3 loop
					for J in 0 to 3 loop
						LED_DATA_R(I*4+J) <= data_reg(I+28)(6-J*2);
						LED_DATA_G(I*4+J) <= data_reg(I+28)(6-J*2+1);
					end loop;
				end loop;
				LED_SEL <= "0000000100000000";
			when "1000" =>
				for I in 0 to 3 loop
					for J in 0 to 3 loop
						LED_DATA_R(I*4+J) <= data_reg(I+32)(6-J*2);
						LED_DATA_G(I*4+J) <= data_reg(I+32)(6-J*2+1);
					end loop;
				end loop;
				LED_SEL <= "0000000010000000";
			when "1001" =>
				for I in 0 to 3 loop
					for J in 0 to 3 loop
						LED_DATA_R(I*4+J) <= data_reg(I+36)(6-J*2);
						LED_DATA_G(I*4+J) <= data_reg(I+36)(6-J*2+1);
					end loop;
				end loop;
				LED_SEL <= "0000000001000000";
			when "1010" =>
				for I in 0 to 3 loop
					for J in 0 to 3 loop
						LED_DATA_R(I*4+J) <= data_reg(I+40)(6-J*2);
						LED_DATA_G(I*4+J) <= data_reg(I+40)(6-J*2+1);
					end loop;
				end loop;
				LED_SEL <= "0000000000100000";
			when "1011" =>
				for I in 0 to 3 loop
					for J in 0 to 3 loop
						LED_DATA_R(I*4+J) <= data_reg(I+44)(6-J*2);
						LED_DATA_G(I*4+J) <= data_reg(I+44)(6-J*2+1);
					end loop;
				end loop;
				LED_SEL <= "0000000000010000";
			when "1100" =>
				for I in 0 to 3 loop
					for J in 0 to 3 loop
						LED_DATA_R(I*4+J) <= data_reg(I+48)(6-J*2);
						LED_DATA_G(I*4+J) <= data_reg(I+48)(6-J*2+1);
					end loop;
				end loop;
				LED_SEL <= "0000000000001000";
			when "1101" =>
				for I in 0 to 3 loop
					for J in 0 to 3 loop
						LED_DATA_R(I*4+J) <= data_reg(I+52)(6-J*2);
						LED_DATA_G(I*4+J) <= data_reg(I+52)(6-J*2+1);
					end loop;
				end loop;
				LED_SEL <= "0000000000000100";
			when "1110" =>
				for I in 0 to 3 loop
					for J in 0 to 3 loop
						LED_DATA_R(I*4+J) <= data_reg(I+56)(6-J*2);
						LED_DATA_G(I*4+J) <= data_reg(I+56)(6-J*2+1);
					end loop;
				end loop;
				LED_SEL <= "0000000000000010";
			when "1111" =>
				for I in 0 to 3 loop
					for J in 0 to 3 loop
						LED_DATA_R(I*4+J) <= data_reg(I+60)(6-J*2);
						LED_DATA_G(I*4+J) <= data_reg(I+60)(6-J*2+1);
					end loop;
				end loop;
				LED_SEL <= "0000000000000001";
			when others =>
				LED_DATA_R <= "XXXXXXXXXXXXXXXX";
				LED_DATA_G <= "XXXXXXXXXXXXXXXX";
				LED_SEL    <= "XXXXXXXXXXXXXXXX";
		end case;
	end process;

	BUZZ_OUT <= not BUZZ_IN;
	
	SW_OUT(6 downto 0) <= SW_IN(6 downto 0);

end RTL;
