-----------------------------------------------------------
-- FPGA Training Board(B6101)
-- Demo-01
-----------------------------------------------------------
-- File   : demo_01_top.vhd
-- Date   : 2007-09-26
-- Family : Cyclone
-- Device : EP1C3T144C8
--
-- Designed by TOYO-LINX Co.,Ltd. / yKikuchi
-----------------------------------------------------------

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;

entity demo_01_top is
	port(
		CLK			:	in std_logic;
		LED_SEL		:	out std_logic_vector(15 downto 0);
		LED_DATA_R	:	out std_logic_vector(0 to 15);
		LED_DATA_G	:	out std_logic_vector(0 to 15);
		SW			:	in std_logic_vector(7 downto 1);
		BUZZ		:	out std_logic;
		LED1A		:	out std_logic;
		LED1B		:	out std_logic;
		LED1C		:	out std_logic;
		LED1D		:	out std_logic;
		LED1E		:	out std_logic;
		LED1F		:	out std_logic;
		LED1G		:	out std_logic;
		LED1DP		:	out std_logic;
		LED2A		:	out std_logic;
		LED2B		:	out std_logic;
		LED2C		:	out std_logic;
		LED2D		:	out std_logic;
		LED2E		:	out std_logic;
		LED2F		:	out std_logic;
		LED2G		:	out std_logic;
		LED2DP		:	out std_logic
	);
end demo_01_top;

architecture RTL of demo_01_top is

	signal div_cnt		:	std_logic_vector(31 downto 0);
	signal div_clk		:	std_logic;
	signal scan_cnt		:	std_logic_vector(3 downto 0);
	signal shift_clk	:	std_logic;
	signal direction	:	std_logic_vector(1 downto 0) := "00";
	signal buzz_osc		:	std_logic_vector(6 downto 0);
	signal count_clk	:	std_logic;
	signal counter0		:	std_logic_vector(3 downto 0);
	signal counter1		:	std_logic_vector(3 downto 0);
	signal count0_en	:	std_logic := '1';
	signal count1_en	:	std_logic := '1';
	signal led1			:	std_logic_vector(7 downto 0);
	signal led2			:	std_logic_vector(7 downto 0);
	
	-----------------------------------------------------------Green-----------------Red---------
	signal led_reg_F	:	std_logic_vector(0 to 31) := "0000000000000000" & "1110000000011111";
	signal led_reg_E	:	std_logic_vector(0 to 31) := "0000000000000000" & "1100000000001111";
	signal led_reg_D	:	std_logic_vector(0 to 31) := "0000000000000000" & "1000000000000111";
	signal led_reg_C	:	std_logic_vector(0 to 31) := "0001111110000000" & "0001111110000000";
	signal led_reg_B	:	std_logic_vector(0 to 31) := "0011111111000000" & "0010010001000000";
	signal led_reg_A	:	std_logic_vector(0 to 31) := "0111111111100000" & "0100010000100000";
	signal led_reg_9	:	std_logic_vector(0 to 31) := "1111111111110000" & "1000010000010000";
	signal led_reg_8	:	std_logic_vector(0 to 31) := "1111111111111100" & "1111111111111100";
	signal led_reg_7	:	std_logic_vector(0 to 31) := "1111111111111110" & "1100111111100110";
	signal led_reg_6	:	std_logic_vector(0 to 31) := "1111111111111110" & "1000011111000010";
	signal led_reg_5	:	std_logic_vector(0 to 31) := "0111100000111100" & "0000000000000000";
	signal led_reg_4	:	std_logic_vector(0 to 31) := "0011000000011000" & "0000000000000000";
	signal led_reg_3	:	std_logic_vector(0 to 31) := "0000000000000000" & "1111111111111111";
	signal led_reg_2	:	std_logic_vector(0 to 31) := "0000000000000000" & "0000000000000000";
	signal led_reg_1	:	std_logic_vector(0 to 31) := "0011001100110011" & "0000000000000000";
	signal led_reg_0	:	std_logic_vector(0 to 31) := "0011001100110011" & "0000000000000000";

begin
	process(CLK)
	begin
		if(CLK'event and CLK='1') then
			div_cnt <= div_cnt + 1;
		end if;
	end process;
	div_clk <= div_cnt(11);
	shift_clk <= div_cnt(19);
	buzz_osc <= div_cnt(15 downto 9);
	count_clk <= div_cnt(18);
	
	process(div_clk)
	begin
		if(div_clk'event and div_clk='1') then
			scan_cnt <= scan_cnt + 1;
		end if;
	end process;

	process(scan_cnt)
	begin
		case scan_cnt is
			when "1111" =>
				LED_DATA_R <= led_reg_F(16 to 31);
				LED_DATA_G <= led_reg_F( 0 to 15);
				LED_SEL    <= "1000000000000000";
			when "1110" =>
				LED_DATA_R <= led_reg_E(16 to 31);
				LED_DATA_G <= led_reg_E( 0 to 15);
				LED_SEL    <= "0100000000000000";
			when "1101" =>
				LED_DATA_R <= led_reg_D(16 to 31);
				LED_DATA_G <= led_reg_D( 0 to 15);
				LED_SEL    <= "0010000000000000";
			when "1100" =>
				LED_DATA_R <= led_reg_C(16 to 31);
				LED_DATA_G <= led_reg_C( 0 to 15);
				LED_SEL    <= "0001000000000000";
			when "1011" =>
				LED_DATA_R <= led_reg_B(16 to 31);
				LED_DATA_G <= led_reg_B( 0 to 15);
				LED_SEL    <= "0000100000000000";
			when "1010" =>
				LED_DATA_R <= led_reg_A(16 to 31);
				LED_DATA_G <= led_reg_A( 0 to 15);
				LED_SEL    <= "0000010000000000";
			when "1001" =>
				LED_DATA_R <= led_reg_9(16 to 31);
				LED_DATA_G <= led_reg_9( 0 to 15);
				LED_SEL    <= "0000001000000000";
			when "1000" =>
				LED_DATA_R <= led_reg_8(16 to 31);
				LED_DATA_G <= led_reg_8( 0 to 15);
				LED_SEL    <= "0000000100000000";
			when "0111" =>
				LED_DATA_R <= led_reg_7(16 to 31);
				LED_DATA_G <= led_reg_7( 0 to 15);
				LED_SEL    <= "0000000010000000";
			when "0110" =>
				LED_DATA_R <= led_reg_6(16 to 31);
				LED_DATA_G <= led_reg_6( 0 to 15);
				LED_SEL    <= "0000000001000000";
			when "0101" =>
				LED_DATA_R <= led_reg_5(16 to 31);
				LED_DATA_G <= led_reg_5( 0 to 15);
				LED_SEL    <= "0000000000100000";
			when "0100" =>
				LED_DATA_R <= led_reg_4(16 to 31);
				LED_DATA_G <= led_reg_4( 0 to 15);
				LED_SEL    <= "0000000000010000";
			when "0011" =>
				LED_DATA_R <= led_reg_3(16 to 31);
				LED_DATA_G <= led_reg_3( 0 to 15);
				LED_SEL    <= "0000000000001000";
			when "0010" =>
				LED_DATA_R <= led_reg_2(16 to 31);
				LED_DATA_G <= led_reg_2( 0 to 15);
				LED_SEL    <= "0000000000000100";
			when "0001" =>
				LED_DATA_R <= led_reg_1(16 to 31);
				LED_DATA_G <= led_reg_1( 0 to 15);
				LED_SEL    <= "0000000000000010";
			when "0000" =>
				LED_DATA_R <= led_reg_0(16 to 31);
				LED_DATA_G <= led_reg_0( 0 to 15);
				LED_SEL    <= "0000000000000001";
			when others =>
				LED_DATA_R <= "0000000000000000";
				LED_DATA_G <= "0000000000000000";
				LED_SEL    <= "0000000000000000";
		end case;
	end process;

	process(shift_clk)
	begin
		if(shift_clk'event and shift_clk='1') then
			if(direction="00") then
				for I in 15 downto 1 loop
					led_reg_F(I+16) <= led_reg_F(I+15);
					led_reg_E(I+16) <= led_reg_E(I+15);
					led_reg_D(I+16) <= led_reg_D(I+15);
					led_reg_C(I+16) <= led_reg_C(I+15);
					led_reg_B(I+16) <= led_reg_B(I+15);
					led_reg_A(I+16) <= led_reg_A(I+15);
					led_reg_9(I+16) <= led_reg_9(I+15);
					led_reg_8(I+16) <= led_reg_8(I+15);
					led_reg_7(I+16) <= led_reg_7(I+15);
					led_reg_6(I+16) <= led_reg_6(I+15);
					led_reg_5(I+16) <= led_reg_5(I+15);
					led_reg_4(I+16) <= led_reg_4(I+15);
					led_reg_3(I+16) <= led_reg_3(I+15);
					led_reg_2(I+16) <= led_reg_2(I+15);
					led_reg_1(I+16) <= led_reg_1(I+15);
					led_reg_0(I+16) <= led_reg_0(I+15);
					led_reg_F(I)    <= led_reg_F(I-1);
					led_reg_E(I)    <= led_reg_E(I-1);
					led_reg_D(I)    <= led_reg_D(I-1);
					led_reg_C(I)    <= led_reg_C(I-1);
					led_reg_B(I)    <= led_reg_B(I-1);
					led_reg_A(I)    <= led_reg_A(I-1);
					led_reg_9(I)    <= led_reg_9(I-1);
					led_reg_8(I)    <= led_reg_8(I-1);
					led_reg_7(I)    <= led_reg_7(I-1);
					led_reg_6(I)    <= led_reg_6(I-1);
					led_reg_5(I)    <= led_reg_5(I-1);
					led_reg_4(I)    <= led_reg_4(I-1);
					led_reg_3(I)    <= led_reg_3(I-1);
					led_reg_2(I)    <= led_reg_2(I-1);
					led_reg_1(I)    <= led_reg_1(I-1);
					led_reg_0(I)    <= led_reg_0(I-1);
				end loop;
				led_reg_F(16) <= led_reg_F(31);
				led_reg_E(16) <= led_reg_E(31);
				led_reg_D(16) <= led_reg_D(31);
				led_reg_C(16) <= led_reg_C(31);
				led_reg_B(16) <= led_reg_B(31);
				led_reg_A(16) <= led_reg_A(31);
				led_reg_9(16) <= led_reg_9(31);
				led_reg_8(16) <= led_reg_8(31);
				led_reg_7(16) <= led_reg_7(31);
				led_reg_6(16) <= led_reg_6(31);
				led_reg_5(16) <= led_reg_5(31);
				led_reg_4(16) <= led_reg_4(31);
				led_reg_3(16) <= led_reg_3(31);
				led_reg_2(16) <= led_reg_2(31);
				led_reg_1(16) <= led_reg_1(31);
				led_reg_0(16) <= led_reg_0(31);
				led_reg_F( 0) <= led_reg_F(15);
				led_reg_E( 0) <= led_reg_E(15);
				led_reg_D( 0) <= led_reg_D(15);
				led_reg_C( 0) <= led_reg_C(15);
				led_reg_B( 0) <= led_reg_B(15);
				led_reg_A( 0) <= led_reg_A(15);
				led_reg_9( 0) <= led_reg_9(15);
				led_reg_8( 0) <= led_reg_8(15);
				led_reg_7( 0) <= led_reg_7(15);
				led_reg_6( 0) <= led_reg_6(15);
				led_reg_5( 0) <= led_reg_5(15);
				led_reg_4( 0) <= led_reg_4(15);
				led_reg_3( 0) <= led_reg_3(15);
				led_reg_2( 0) <= led_reg_2(15);
				led_reg_1( 0) <= led_reg_1(15);
				led_reg_0( 0) <= led_reg_0(15);
			elsif(direction="01") then
				for I in 0 to 14 loop
					led_reg_F(I+16) <= led_reg_F(I+17);
					led_reg_E(I+16) <= led_reg_E(I+17);
					led_reg_D(I+16) <= led_reg_D(I+17);
					led_reg_C(I+16) <= led_reg_C(I+17);
					led_reg_B(I+16) <= led_reg_B(I+17);
					led_reg_A(I+16) <= led_reg_A(I+17);
					led_reg_9(I+16) <= led_reg_9(I+17);
					led_reg_8(I+16) <= led_reg_8(I+17);
					led_reg_7(I+16) <= led_reg_7(I+17);
					led_reg_6(I+16) <= led_reg_6(I+17);
					led_reg_5(I+16) <= led_reg_5(I+17);
					led_reg_4(I+16) <= led_reg_4(I+17);
					led_reg_3(I+16) <= led_reg_3(I+17);
					led_reg_2(I+16) <= led_reg_2(I+17);
					led_reg_1(I+16) <= led_reg_1(I+17);
					led_reg_0(I+16) <= led_reg_0(I+17);
					led_reg_F(I)    <= led_reg_F(I+1);
					led_reg_E(I)    <= led_reg_E(I+1);
					led_reg_D(I)    <= led_reg_D(I+1);
					led_reg_C(I)    <= led_reg_C(I+1);
					led_reg_B(I)    <= led_reg_B(I+1);
					led_reg_A(I)    <= led_reg_A(I+1);
					led_reg_9(I)    <= led_reg_9(I+1);
					led_reg_8(I)    <= led_reg_8(I+1);
					led_reg_7(I)    <= led_reg_7(I+1);
					led_reg_6(I)    <= led_reg_6(I+1);
					led_reg_5(I)    <= led_reg_5(I+1);
					led_reg_4(I)    <= led_reg_4(I+1);
					led_reg_3(I)    <= led_reg_3(I+1);
					led_reg_2(I)    <= led_reg_2(I+1);
					led_reg_1(I)    <= led_reg_1(I+1);
					led_reg_0(I)    <= led_reg_0(I+1);
				end loop;
				led_reg_F(31) <= led_reg_F(16);
				led_reg_E(31) <= led_reg_E(16);
				led_reg_D(31) <= led_reg_D(16);
				led_reg_C(31) <= led_reg_C(16);
				led_reg_B(31) <= led_reg_B(16);
				led_reg_A(31) <= led_reg_A(16);
				led_reg_9(31) <= led_reg_9(16);
				led_reg_8(31) <= led_reg_8(16);
				led_reg_7(31) <= led_reg_7(16);
				led_reg_6(31) <= led_reg_6(16);
				led_reg_5(31) <= led_reg_5(16);
				led_reg_4(31) <= led_reg_4(16);
				led_reg_3(31) <= led_reg_3(16);
				led_reg_2(31) <= led_reg_2(16);
				led_reg_1(31) <= led_reg_1(16);
				led_reg_0(31) <= led_reg_0(16);
				led_reg_F(15) <= led_reg_F(0);
				led_reg_E(15) <= led_reg_E(0);
				led_reg_D(15) <= led_reg_D(0);
				led_reg_C(15) <= led_reg_C(0);
				led_reg_B(15) <= led_reg_B(0);
				led_reg_A(15) <= led_reg_A(0);
				led_reg_9(15) <= led_reg_9(0);
				led_reg_8(15) <= led_reg_8(0);
				led_reg_7(15) <= led_reg_7(0);
				led_reg_6(15) <= led_reg_6(0);
				led_reg_5(15) <= led_reg_5(0);
				led_reg_4(15) <= led_reg_4(0);
				led_reg_3(15) <= led_reg_3(0);
				led_reg_2(15) <= led_reg_2(0);
				led_reg_1(15) <= led_reg_1(0);
				led_reg_0(15) <= led_reg_0(0);
			elsif(direction="10") then
				led_reg_F <= led_reg_E;
				led_reg_E <= led_reg_D;
				led_reg_D <= led_reg_C;
				led_reg_C <= led_reg_B;
				led_reg_B <= led_reg_A;
				led_reg_A <= led_reg_9;
				led_reg_9 <= led_reg_8;
				led_reg_8 <= led_reg_7;
				led_reg_7 <= led_reg_6;
				led_reg_6 <= led_reg_5;
				led_reg_5 <= led_reg_4;
				led_reg_4 <= led_reg_3;
				led_reg_3 <= led_reg_2;
				led_reg_2 <= led_reg_1;
				led_reg_1 <= led_reg_0;
				led_reg_0 <= led_reg_F;
			elsif(direction="11") then
				led_reg_F <= led_reg_0;
				led_reg_E <= led_reg_F;
				led_reg_D <= led_reg_E;
				led_reg_C <= led_reg_D;
				led_reg_B <= led_reg_C;
				led_reg_A <= led_reg_B;
				led_reg_9 <= led_reg_A;
				led_reg_8 <= led_reg_9;
				led_reg_7 <= led_reg_8;
				led_reg_6 <= led_reg_7;
				led_reg_5 <= led_reg_6;
				led_reg_4 <= led_reg_5;
				led_reg_3 <= led_reg_4;
				led_reg_2 <= led_reg_3;
				led_reg_1 <= led_reg_2;
				led_reg_0 <= led_reg_1;
			else
				led_reg_F <= led_reg_F;
				led_reg_E <= led_reg_E;
				led_reg_D <= led_reg_D;
				led_reg_C <= led_reg_C;
				led_reg_B <= led_reg_B;
				led_reg_A <= led_reg_A;
				led_reg_9 <= led_reg_9;
				led_reg_8 <= led_reg_8;
				led_reg_7 <= led_reg_7;
				led_reg_6 <= led_reg_6;
				led_reg_5 <= led_reg_5;
				led_reg_4 <= led_reg_4;
				led_reg_3 <= led_reg_3;
				led_reg_2 <= led_reg_2;
				led_reg_1 <= led_reg_1;
				led_reg_0 <= led_reg_0;
			end if;
		end if;
	end process;

	process(shift_clk)
	begin
		if(shift_clk'event and shift_clk='1') then
			if   (SW(4)='0')then
				direction <= "00";
			elsif(SW(3)='0')then
				direction <= "01";
			elsif(SW(2)='0')then
				direction <= "11";
			elsif(SW(1)='0')then
				direction <= "10";
			end if;
		end if;
	end process;

	process(SW)
	begin
		if   (SW(7)='0')then
			BUZZ <= buzz_osc(6);
		elsif(SW(6)='0')then
			BUZZ <= buzz_osc(5);
		elsif(SW(5)='0')then
			BUZZ <= buzz_osc(4);
		elsif(SW(4)='0')then
			BUZZ <= buzz_osc(3);
		elsif(SW(3)='0')then
			BUZZ <= buzz_osc(2);
		elsif(SW(2)='0')then
			BUZZ <= buzz_osc(1);
		elsif(SW(1)='0')then
			BUZZ <= buzz_osc(0);
		else
			BUZZ <= '0';
		end if;
	end process;

	process(count_clk)
	begin
		if(count_clk'event and count_clk='1') then
			if   (SW(7)='0')then
				count0_en <= '1';
				count1_en <= '1';
			elsif(SW(6)='0')then
				count0_en <= '0';
			elsif(SW(5)='0')then
				count1_en <= '0';
			end if;
		end if;
	end process;

	process(count_clk)
	begin
		if(count_clk'event and count_clk='1') then
			if(count0_en='1')then
				counter0 <= counter0 + 1;
			end if;
			if(count1_en='1')then
				counter1 <= counter1 + 1;
			end if;
		end if;
	end process;

	process(counter0,counter1)
	begin
		case counter0 is
			when "0000" =>
				led2 <= "00111111";
			when "0001" =>
				led2 <= "00000110";
			when "0010" =>
				led2 <= "01011011";
			when "0011" =>
				led2 <= "01001111";
			when "0100" =>
				led2 <= "01100110";
			when "0101" =>
				led2 <= "01101101";
			when "0110" =>
				led2 <= "01111101";
			when "0111" =>
				led2 <= "00000111";
			when "1000" =>
				led2 <= "01111111";
			when "1001" =>
				led2 <= "01101111";
			when "1010" =>
				led2 <= "01110111";
			when "1011" =>
				led2 <= "01111100";
			when "1100" =>
				led2 <= "00111001";
			when "1101" =>
				led2 <= "01011110";
			when "1110" =>
				led2 <= "01111001";
			when "1111" =>
				led2 <= "01110001";
			when others =>
				led2 <= "00000000";
		end case;
		case counter1 is
			when "0000" =>
				led1 <= "00111111";
			when "0001" =>
				led1 <= "00000110";
			when "0010" =>
				led1 <= "01011011";
			when "0011" =>
				led1 <= "01001111";
			when "0100" =>
				led1 <= "01100110";
			when "0101" =>
				led1 <= "01101101";
			when "0110" =>
				led1 <= "01111101";
			when "0111" =>
				led1 <= "00000111";
			when "1000" =>
				led1 <= "01111111";
			when "1001" =>
				led1 <= "01101111";
			when "1010" =>
				led1 <= "01110111";
			when "1011" =>
				led1 <= "01111100";
			when "1100" =>
				led1 <= "00111001";
			when "1101" =>
				led1 <= "01011110";
			when "1110" =>
				led1 <= "01111001";
			when "1111" =>
				led1 <= "01110001";
			when others =>
				led1 <= "00000000";
		end case;
	end process;

	LED1A  <= led1(0);
	LED1B  <= led1(1);
	LED1C  <= led1(2);
	LED1D  <= led1(3);
	LED1E  <= led1(4);
	LED1F  <= led1(5);
	LED1G  <= led1(6);
	LED1DP <= led1(7);
	LED2A  <= led2(0);
	LED2B  <= led2(1);
	LED2C  <= led2(2);
	LED2D  <= led2(3);
	LED2E  <= led2(4);
	LED2F  <= led2(5);
	LED2G  <= led2(6);
	LED2DP <= led2(7);

end RTL;
